import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;
import java.util.*;

/**
 * コントロールクラス.
 */
public class KCtrlPZL extends KCtrl implements KConstant {

	/**
	 * フェーズ.
	 */
	private final int PHASE_INIT		= 0;
	private final int PHASE_TITLE		= 1;
	private final int PHASE_STAGE_START	= 2;
	private final int PHASE_RUN			= 3;
	private final int PHASE_FAIL		= 4;
	private final int PHASE_END			= 5;
	private final int PHASE_STOP		= -1;

	/**
	 * 落下速度.
	 */
	private final int SPEED				= 16;

	/**
	 * 追加ブロックを追加するまでの時間.
	 */
	private final int TIME_ADD_BLOCK	= 2;

	/**
	 * パネル.
	 */
	private		KPanel				panel				= null;

	/**
	 * イメージ.
	 */
	private		BufferedImage		imageBackground		= null;
	private		BufferedImage		imageBlock00_01		= null;
	private		BufferedImage		imageBlock00_02		= null;
	private		BufferedImage		imageBlock00_03		= null;
	private		BufferedImage		imageBlock01		= null;
	private		BufferedImage		imageBlock02		= null;
	private		BufferedImage		imageBlock03		= null;
	private		BufferedImage		imageBlock04		= null;
	private		BufferedImage		imageBlock05		= null;
	private		BufferedImage		imageBlock06		= null;
	private		BufferedImage		imageBlock07		= null;
	private		BufferedImage		imageBlockAA		= null;
	private		BufferedImage		imageKakomuX		= null;
	private		BufferedImage		imageKakomuY		= null;

	/**
	 * スプライト.
	 * 全体的に表示等されるスプライトは、ここで管理する。
	 */
	private		KSpriteBackground	spriteBackground	= null;

	/**
	 * 移動用スプライトブロック.
	 */
	private		KSpriteKakomuX		sprKakomuX			= null;
	private		KSpriteKakomuY		sprKakomuY			= null;

	/**
	 * テキスト.
	 */
	private		KTextTitle			kTextTitle			= null;
	private		KText				kTextStart			= null;
	private		KText				kTextBy				= null;
	private		KText				kTextEnd			= null;
	private		KText				kTextScoreHigh		= null;
	private		KText				kTextScore			= null;
	private		KText				kTextStage			= null;
	private		KText				kTextLimit			= null;
	private		KText				kTextMessage		= null;

	/**
	 * スプライトブロック一覧.
	 */
	private		KSpriteBlock01[][]	sprBlocks			= new KSpriteBlock01[BLOCK_ROW_COUNT][BLOCK_COL_COUNT];

	/**
	 * 追加ブロック.
	 */
	private		int[]				addBlocks			= new int[BLOCK_COL_COUNT];

	/**
	 * 落下中スプライトブロック一覧.
	 */
	private		ArrayList<KSpriteBlock01>	sprFallBlockLst		= new ArrayList<KSpriteBlock01>(100);

	/**
	 * 削除時間一覧.
	 */
	private		ArrayList<String>		timeLst				= new ArrayList<String>();

	/**
	 * 削除スプライトブロック一覧一覧.
	 */
	private		ArrayList<ArrayList<KSpriteBlock01>>	deleteSprBlockLstLst		= new ArrayList<ArrayList<KSpriteBlock01>>();

	/**
	 * フェーズ.
	 */
	private		int					phase				= PHASE_INIT;

	/**
	 * スコア.
	 */
	private		int					score				= 0;

	/**
	 * ハイスコア.
	 */
	private		int					scoreHigh			= 10000;

	/**
	 * ステージ.
	 */
	private		int					stage				= 1;

	/**
	 * 時間.
	 */
	private		int					time				= 0;

	/**
	 * 制限時間.
	 */
	private		int					limit				= 0;

	/**
	 * マウスアダプタ.
	 */
	private		KCPMouseAdapter		kcpMouseAdapter		= null;

	/**
	 * 選んでいるブロック位置.
	 */
	private		int					selectRow0			= -1;
	private		int					selectCol0			= -1;
	private		int					selectRow1			= -1;
	private		int					selectCol1			= -1;

	/**
	 * 選んでいるマウス位置.
	 */
	private		int					x0					= -1;
	private		int					y0					= -1;

	/**
	 * チェックブロック一覧.
	 */
	ArrayList<KSpriteBlock01> checkLst = new ArrayList<KSpriteBlock01>();

	/**
	 * 音楽.
	 */
	private		KSoundMidi				midiTitle		= null;
	private		KSoundMidi				midiGame		= null;

	private		KSoundWave				waveClick		= null;
	private		KSoundWave				waveDelete		= null;

	/**
	 * コンストラクタ.
	 * @param con コンテナ
	 */
	public KCtrlPZL(Container con){

		super(con);

		try{

			// パネルを生成
			panel = new KPanel(this);

			// パネルのサイズを設定
			panel.setSize(KConstant.WIDTH, KConstant.HEIGHT);
			panel.setPreferredSize(new Dimension(KConstant.WIDTH, KConstant.HEIGHT));

			// スプライト層の数を設定
			panel.setLayoutCount(LAYOUT_COUNT);

			// イメージを取得
			imageBackground = KImage.getImage("imageBackground.jpg");
			imageBlock00_01 = KImage.getImage("imageBlock00_01.gif");
			imageBlock00_02 = KImage.getImage("imageBlock00_02.gif");
			imageBlock00_03 = KImage.getImage("imageBlock00_03.gif");
			imageBlock01 = KImage.getImage("imageBlock01.gif");
			imageBlock02 = KImage.getImage("imageBlock02.gif");
			imageBlock03 = KImage.getImage("imageBlock03.gif");
			imageBlock04 = KImage.getImage("imageBlock04.gif");
			imageBlock05 = KImage.getImage("imageBlock05.gif");
			imageBlock06 = KImage.getImage("imageBlock06.gif");
			imageBlock07 = KImage.getImage("imageBlock07.gif");
			imageBlockAA = KImage.getImage("imageBlockAA.gif");
			imageKakomuX = KImage.getImage("imageKakomuX.gif");
			imageKakomuY = KImage.getImage("imageKakomuY.gif");

			// イメージをスプライトに登録する
			KSpriteBackground.setImage(KSpriteBackground.class, imageBackground);
			KSpriteBlock00.setImage(KSpriteBlock00.class, new BufferedImage[]{imageBlock00_01, imageBlock00_02, imageBlock00_03});
			KSpriteBlock01.setImage(KSpriteBlock01.class, new BufferedImage[]{imageBlock01, imageBlock02, imageBlock03, imageBlock04, imageBlock05, imageBlock06, imageBlock07});
			KSpriteBlockAA.setImage(KSpriteBlockAA.class, imageBlockAA);
			KSpriteKakomuX.setImage(KSpriteKakomuX.class, imageKakomuX);
			KSpriteKakomuY.setImage(KSpriteKakomuY.class, imageKakomuY);

			// 背景生成
			spriteBackground = new KSpriteBackground(panel);

			//
			// 各テキストを生成
			//

			kTextTitle = new KTextTitle(panel, 0, 0);

			kTextStart = new KText(panel, 176, 400);
			kTextStart.setFont(new Font("HG丸ｺﾞｼｯｸM-PRO", Font.PLAIN, 16));
			kTextStart.setText("CLICK MOUSE BUTTON");

			kTextBy = new KText(panel, 200, 480);
			kTextBy.setFont(new Font("ＭＳ ゴシック", Font.PLAIN, 12));
			kTextBy.setText("PROGRAMED  BY  KIN");

			kTextEnd = new KText(panel, 208, 100);
			kTextEnd.setFont(new Font("HG丸ｺﾞｼｯｸM-PRO", Font.BOLD, 24));
			kTextEnd.setText("GAME OVER");

			kTextScoreHigh = new KText(panel, 50, 20);
			panel.addText("HIGH-SCORE", kTextScoreHigh);

			kTextScore = new KText(panel, 200, 20);
			panel.addText("SCORE", kTextScore);

			kTextStage = new KText(panel, 350, 20);
			panel.addText("STAGE", kTextStage);

			kTextLimit = new KText(panel, 350, 36);
			panel.addText("LIMIT", kTextLimit);

			kTextMessage = new KText(panel, 240, 100);
			kTextMessage.setFont(new Font("HG丸ｺﾞｼｯｸM-PRO", Font.BOLD, 24));

			// 囲むスプライト生成
			sprKakomuX = new KSpriteKakomuX(panel);
			sprKakomuY = new KSpriteKakomuY(panel);

			// マウスアダプタ生成
			kcpMouseAdapter = new KCPMouseAdapter();

			// マウスアダプタをパネルに登録
			panel.addMouseListener(kcpMouseAdapter);
			panel.addMouseMotionListener(kcpMouseAdapter);

			// 音楽オブジェクトを生成
			waveClick = new KSoundWave(this, "sound\\Click.WAV", false);
			waveDelete = new KSoundWave(this, "sound\\ERASE.WAV", false);

		// エラーの場合、
		}catch(Exception ex){
			// スタックトレース出力
			ex.printStackTrace();
			// エラーを表示
			JOptionPane.showMessageDialog(null, ex.toString());
		}

	} // end KCtrlPZL

	/**
	 * 開始.
	 * <pre>
	 * 当オブジェクトの処理を開始します。
	 * </pre>
	 */
	public void start(){

		// パネルにフォーカスを当てる
		panel.requestFocus();

		// パネルを開始する
		panel.start();

		// 初期化フェーズへ
		this.phase = PHASE_INIT;

	} // end start

	/**
	 * 実行.
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * </pre>
	 */
	synchronized public void run(){

		// フェーズ分け
		switch(this.phase){

			// 初期化フェーズ
			case PHASE_INIT:

				// スコア
				score = 0;

				// ステージ
				stage = 1;

				// 時間初期化
				time = 0;

				// 制限時間初期化
				limit = LIMIT_INIT;

				// ブロックのスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_SENTAKU);
				panel.initKSpriteLstInLayout(LAYOUT_BLK0);
				panel.initKSpriteLstInLayout(LAYOUT_BLK1);

				// 背景のスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_HAIKEI0);
				panel.initKSpriteLstInLayout(LAYOUT_HAIKEI1);
				panel.initKSpriteLstInLayout(LAYOUT_HAIKEI2);

				// スプライトグループ初期化
				panel.initSpriteGroup();

				// スプライトブロック一覧初期化.
				sprBlocks = new KSpriteBlock01[BLOCK_ROW_COUNT][BLOCK_COL_COUNT];
				// 追加ブロック初期化
				addBlocks = new int[BLOCK_COL_COUNT];
				// 落下中スプライトブロック一覧初期化
				sprFallBlockLst		= new ArrayList<KSpriteBlock01>(100);
				// 削除時間一覧初期化
				timeLst				= new ArrayList<String>();
				// 削除スプライトブロック一覧一覧初期化
				deleteSprBlockLstLst		= new ArrayList<ArrayList<KSpriteBlock01>>();
				// チェックブロック一覧初期化
				checkLst = new ArrayList<KSpriteBlock01>();

				// 背景を登録
				panel.addSprite(LAYOUT_HAIKEI0, spriteBackground);

				// タイトル表示
				panel.addText("TITLE", kTextTitle);
				panel.addText("START", kTextStart);
				panel.addText("BY", kTextBy);

				// 各テキストを最新化
				kTextScoreHigh.setText("HIGH-SCORE : " + Integer.toString(scoreHigh));

				// スコア初期化
				kTextScore.setText("SCORE : " + Integer.toString(score));

				// ステージ初期化
				kTextStage.setText("STAGE : " + Integer.toString(stage));

				// 時間初期化
				kTextLimit.setText("LIMIT : " + Integer.toString(limit / 60));

				// 音楽オブジェクトを生成
				midiTitle = new KSoundMidi(this, "sound\\PZL_Title.mid", true);
				midiGame = new KSoundMidi(this, "sound\\Pazz.MID", true);

				//
				// 壁を作成
				//

				// 縦ループ
				for(int row = 0; row < 17; row++){

					KSpriteBlockAA kSpriteBlockAA = new KSpriteBlockAA(panel);
					panel.addSprite(LAYOUT_HAIKEI1, kSpriteBlockAA);
					kSpriteBlockAA.setY(row * 32);

					kSpriteBlockAA = new KSpriteBlockAA(panel);
					panel.addSprite(LAYOUT_HAIKEI1, kSpriteBlockAA);
					kSpriteBlockAA.setX(512);
					kSpriteBlockAA.setY(row * 32);

				} // ewd for 縦ループ

				// 横ループ
				for(int col = 0; col < 17; col++){

					KSpriteBlockAA kSpriteBlockAA = new KSpriteBlockAA(panel);
					panel.addSprite(LAYOUT_HAIKEI1, kSpriteBlockAA);
					kSpriteBlockAA.setX(col * 32);
					kSpriteBlockAA.setY(512);

				} // ewd for 横ループ

				// 音楽をスタート
				midiTitle.start();

				// タイトル表示フェーズへ
				this.phase = PHASE_TITLE;

				break;

			// タイトル表示フェーズ
			case PHASE_TITLE:

				break;

			// ステージ開始フェーズ
			case PHASE_STAGE_START:

				time++;

				// READY
				if(time == 1){

					// タイトルを削除
					panel.removeText("TITLE");
					panel.removeText("START");
					panel.removeText("BY");

					// ブロック初期化
					initBlocks();

					// 場所修正
					kTextMessage.setX(216);

					// READY
					kTextMessage.setText("READY");

					// READY表示
					panel.addText("MESSAGE", kTextMessage);

				// GO!
				}else if(time == 120){

					// 場所修正
					kTextMessage.setX(240);

					// GO!
					kTextMessage.setText("GO!");

				// 本当にGO!
				}else if(time == 180){

					// GO!削除
					panel.removeText("MESSAGE");

					// 音楽をスタート
					midiGame.start();

					// 時間初期化
					time = 0;

					// 通常フェーズへ
					this.phase = PHASE_RUN;

				}

				break;

			// 通常フェーズ
			case PHASE_RUN:

				// 時間を進める
				time++;

				//
				// 追加ブロックを落下させる
				//

				// 時間がきた場合、
				if(time % TIME_ADD_BLOCK == 0){

					// 列数分、ループ
					for(int col = 0; col < BLOCK_COL_COUNT; col++){

						// 追加ブロックがある場合、
						if(addBlocks[col] > 0){

							// 追加ブロックをひとつ減らす
							addBlocks[col]--;

							// 新しいブロック色を取得
							int block = getNewBlock();

							// 落下中スプライトブロック生成
							KSpriteBlock01 sprFallBlock = new KSpriteBlock01(panel, block);

							// XY座標設定
							sprFallBlock.setX(col * 32 + 32);
							sprFallBlock.setY(-32);

							// 落下中スプライトブロック一覧に追加
							sprFallBlockLst.add(sprFallBlock);

							// パネルにブロックを追加
							panel.addSprite(LAYOUT_BLK0, sprFallBlock);

						} // end if 追加ブロックがある場合

					} // 列数分、ループ

				} // end if 時間がきた場合

				//
				// 落下中ブロック処理
				//

				// 落下中スプライトブロック一覧の件数分、ループ
				for(int i = 0; i < sprFallBlockLst.size(); i++){

					// 落下中スプライトブロックを取得
					KSpriteBlock01 sprFallBlock = sprFallBlockLst.get(i);

					// XY座標を取得
					int y = sprFallBlock.getY();
					int x = sprFallBlock.getX();

					// 落下させる
					y = y + SPEED;

					// Y座標設定
					sprFallBlock.setY(y);

					// まだ上のほうで落下中の場合、
					if(y < POS_Y){
						// コンティニュー
						continue;
					}

					// 下のブロックがあるかチェックする必要が無い場合、
					if(y % 32 != 0){
						// コンティニュー
						continue;
					}

					// 行列位置取得
					int row = getRow(y);
					int col = getCol(x);

					// 一番下まで来た場合、
					//   または、落下中ブロックの下にブロックがある場合
					if(row == BLOCK_ROW_COUNT - 1 || sprBlocks[row + 1][col] != null){

						// 落下中スプライトブロック一覧より削除
						sprFallBlockLst.remove(i);

						// ループをひとつ戻す
						i--;

						// スプライトブロック一覧に設定
						sprBlocks[row][col] = sprFallBlock;

						// チェックブロック一覧に入っていなければ、
						if(!checkLst.contains(sprFallBlock)){
							// チェックブロック一覧に追加
							this.checkLst.add(sprFallBlock);
							checkLst.add(sprFallBlock);
						}

					} // end if 落下中ブロックの下にブロックがある場合

				} // end for 落下中スプライトブロック一覧の件数分、ループ

				//
				// チェック
				//

				// 落下中スプライトブロックが全て無くなった場合、
				if(sprFallBlockLst.size() == 0){

					// チェックし、削除スプライトブロック一覧を取得
					ArrayList<KSpriteBlock01> deleteSprBlockLst = check(checkLst);

					// 削除ブロックが取得できた場合、
					if(deleteSprBlockLst != null){

						// 削除時間一覧に追加
						timeLst.add("0");

						// 削除スプライトブロック一覧一覧に追加
						deleteSprBlockLstLst.add(deleteSprBlockLst);

					} // end if 削除ブロックが取得できた場合

					// チェックブロックを初期化
					this.checkLst.clear();

				} // end if チェックが必要な場合、

				//
				// 削除処理
				//

				// 時間一覧の件数分、ループ
				for(int i = 0; i < timeLst.size(); i++){

					// 時間を取得
					int time = Integer.parseInt(timeLst.get(i));

					// 時間を＋１
					time++;

					// 時間を設定
					timeLst.set(i, String.valueOf(time));

					// 削除のはじめの時、
					if(time == 1){

						// 音楽をスタート
						waveDelete.start();

					// イメージ変更時間が来た場合、
					}else if(time == 10 || time == 20){

						int deleteStage = 0;
						if(time == 10){
							deleteStage = 1;
						}else{
							deleteStage = 2;
						}

						// 削除一覧を取得
						ArrayList<KSpriteBlock01> deleteSprBlockLst = deleteSprBlockLstLst.get(i);

						// 削除一覧の件数分、ループ
						for(int j = 0; j < deleteSprBlockLst.size(); j++){

							// 削除スプライトブロックを取得
							KSpriteBlock01 sprBlock = deleteSprBlockLst.get(j);

							// 削除段階を設定する
							sprBlock.setDeleteStage(deleteStage);

						} // end for 削除一覧の件数分、ループ

					// スプライト削除時間が来た場合、
					}else if(time == 30){

						// 削除スプライトブロック一覧を取得
						ArrayList<KSpriteBlock01> deleteSprBlockLst = deleteSprBlockLstLst.get(i);

						// 削除スプライトブロック一覧の件数分、ループ
						for(int j = 0; j < deleteSprBlockLst.size(); j++){

							// 削除スプライトブロックを取得
							KSpriteBlock01 sprBlock = deleteSprBlockLst.get(j);

							// スプライトを削除
							sprBlock.removeSprite();

							// 落下中スプライトブロック一覧に登録されている場合、
							if(sprFallBlockLst.indexOf(sprBlock) != -1){

								// 落下中スプライトブロック一覧から削除
								sprFallBlockLst.remove(sprBlock);

							// 落下中スプライトブロック一覧に登録されていない場合、
							// （止まっている場合）
							}else{

								// 表示位置取得
								int y = sprBlock.getY();
								int x = sprBlock.getX();

								// 行列位置取得
								int row = getRow(y);
								int col = getCol(x);

								// スプライトブロック一覧から削除
								sprBlocks[row][col] = null;

								// 上のブロック分、ループ
								for(int row1 = row - 1; row1 >= 0; row1--){

									// スプライトブロックを取得
									sprBlock = sprBlocks[row1][col];

									// スプライトブロックが存在する場合、
									if(sprBlock != null){

										// スプライトブロック一覧より削除
										sprBlocks[row1][col] = null;

										// 落下中スプライトブロック一覧に追加
										sprFallBlockLst.add(sprBlock);

									} // end if スプライトブロックが存在する場合

								} // end for 上のブロック分、ループ

							} // end if 落下中スプライトブロック一覧に登録されていない場合

						} // end for 削除スプライトブロック一覧の件数分、ループ

						// 削除スプライトブロック一覧の件数分、ループ
						for(int j = 0; j < deleteSprBlockLst.size(); j++){

							// 削除スプライトブロックを取得
							KSpriteBlock01 sprBlock = deleteSprBlockLst.get(j);

							// 表示位置取得
							int x = sprBlock.getX();

							// 追加ブロック一覧に追加
							addBlocks[getCol(x)]++;

							// スコアを加算
							addScore(10);

						} // end for 削除スプライトブロック一覧の件数分、ループ

						// 時間一覧から削除
						timeLst.remove(i);

						// 削除スプライトブロック一覧一覧から削除
						deleteSprBlockLstLst.remove(i);

						// ループをひとつ戻す
						i--;

					} // end if スプライト削除時間が来た場合

				} // end for 時間一覧の件数分、ループ

				// 制限時間計算
				limit--;

				// 時間が０以上の場合、
				if(limit >= 0){

					// 制限時間表示
					kTextLimit.setText("LIMIT : " + Integer.toString(limit / 60));

				// 時間がマイナスの場合、
				}else{

					// 制限時間初期化
					limit = 120;

					// 音楽をストップ
					midiGame.stop();

					// 失敗フェーズへ
					this.phase = PHASE_FAIL;

				} // end if 時間がマイナスの場合

				break;

			// 失敗フェーズ
			case PHASE_FAIL:

				// 制限時間計算
				limit--;

				// 時間がマイナスの場合、
				if(limit < 0){

					// 制限時間初期化
					limit = 300;

					// ゲームオーバーを表示
					panel.addText("GAME OVER", kTextEnd);

					// ゲームオーバーフェーズへ
					this.phase = PHASE_END;

				} // end if 時間がマイナスの場合

				break;

			// ゲームオーバーフェーズ
			case PHASE_END:

				// 制限時間計算
				limit--;

				// 時間がマイナスの場合、
				if(limit < 0){

					// ゲームオーバーを消す
					panel.removeText("GAME OVER");

					// 初期化フェーズへ
					this.phase = PHASE_INIT;

				} // end if 時間がマイナスの場合

				break;

			default:
				break;

		} // end switch フェーズ分け

	} // end run

	/**
	 * チェック.
	 * <pre>
	 * 削除できるブロックがある場合は、ArrayListを返却します。
	 * 削除できるブロックがない場合は、nullを返却します。
	 * </pre>
	 * @param checkLst チェックブロック一覧
	 * @return ArrayList<KSpriteBlock01> 削除スプライトブロック一覧
	 */
	private ArrayList<KSpriteBlock01> check(ArrayList<KSpriteBlock01> checkLst){

		// 削除フラグ
		boolean flgDelete = false;

		// 削除ブロック一覧
		int[][] deleteBlocks = new int[BLOCK_ROW_COUNT][BLOCK_COL_COUNT];

		// チェックブロック一覧の件数分、ループ
		for(int i = 0; i < checkLst.size(); i++){

			// チェックブロック取得
			KSpriteBlock01 sprBlock = checkLst.get(i);

			int row = getRow(sprBlock.getY());
			int col = getCol(sprBlock.getX());

			// 既にチェックが終わっている場合、
			if(deleteBlocks[row][col] != CHECK_NO_CHECK){
				// コンティニュー
				continue;
			}

			// 既に削除段階にある場合、
			if(sprBlocks[row][col].getDeleteStage() != -1){
				// コンティニュー
				continue;
			}

			// ブロック色を取得
			int color = sprBlocks[row][col].getColor();

			// 削除候補一覧生成
			ArrayList<int[]> deleteKohoLst	= new ArrayList<int[]>();

			// チェック状況一覧
			int[][] checkedDeletes = new int[BLOCK_ROW_COUNT][BLOCK_COL_COUNT];

			// チェック状況一覧に反映。削除候補とする。
			checkedDeletes[row][col] = CHECK_DELETE;

			// 削除候補一覧に追加
			deleteKohoLst.add(new int[]{row, col});

			// 削除候補一覧の件数分、ループ
			for(int j = 0; j < deleteKohoLst.size(); j++){

				int row0 = deleteKohoLst.get(j)[0];
				int col0 = deleteKohoLst.get(j)[1];

				// 既に削除段階にある場合、
				if(sprBlocks[row0][col0].getDeleteStage() != -1){
					// コンティニュー
					continue;
				}

				// 左

				// 一番左ではない場合、
				if(col0 > 0){

					// 位置情報
					int row1 = row0;
					int col1 = col0 - 1;

					// 未チェック かつ 削除ブロック一覧に登録されていない場合、
					if(checkedDeletes[row1][col1] == CHECK_NO_CHECK && deleteBlocks[row1][col1] == CHECK_NO_CHECK){

						// スプライトブロックが存在しない場合、
						if(sprBlocks[row1][col1] == null){

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 既に削除段階にある場合、
						}else if(sprBlocks[row1][col1].getDeleteStage() != -1){

							// チェック状況一覧に反映。残す。
							// （と、いうか、別のところで消える）
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 同じ色の場合、
						}else if(sprBlocks[row1][col1].getColor() == color){

							// チェック状況一覧に反映。削除候補にする
							checkedDeletes[row1][col1] = CHECK_DELETE;

							// 削除候補一覧に追加
							deleteKohoLst.add(new int[]{row1, col1});

						// 同じ色でない場合、
						}else{

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						}

					} // end if 未チェックの場合

				} // end if 一番左ではない場合

				// 右

				// 一番右ではない場合、
				if(col0 < BLOCK_COL_COUNT - 1){

					// 位置情報
					int row1 = row0;
					int col1 = col0 + 1;

					// 未チェック かつ 削除ブロック一覧に登録されていない場合、
					if(checkedDeletes[row1][col1] == CHECK_NO_CHECK && deleteBlocks[row1][col1] == CHECK_NO_CHECK){

						// スプライトブロックが存在しない場合、
						if(sprBlocks[row1][col1] == null){

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 既に削除段階にある場合、
						}else if(sprBlocks[row1][col1].getDeleteStage() != -1){

							// チェック状況一覧に反映。残す。
							// （と、いうか、別のところで消える）
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 同じ色の場合、
						}else if(sprBlocks[row1][col1].getColor() == color){

							// チェック状況一覧に反映。削除候補にする
							checkedDeletes[row1][col1] = CHECK_DELETE;

							// 削除候補一覧に追加
							deleteKohoLst.add(new int[]{row1, col1});

						// 同じ色でない場合、
						}else{

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						}

					} // end if 未チェックの場合

				} // end if 一番右ではない場合

				// 上

				// 一番上ではない場合、
				if(row0 > 0){

					// 位置情報
					int row1 = row0 - 1;
					int col1 = col0;

					// 未チェック かつ 削除ブロック一覧に登録されていない場合、
					if(checkedDeletes[row1][col1] == CHECK_NO_CHECK && deleteBlocks[row1][col1] == CHECK_NO_CHECK){

						// スプライトブロックが存在しない場合、
						if(sprBlocks[row1][col1] == null){

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 既に削除段階にある場合、
						}else if(sprBlocks[row1][col1].getDeleteStage() != -1){

							// チェック状況一覧に反映。残す。
							// （と、いうか、別のところで消える）
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 同じ色の場合、
						}else if(sprBlocks[row1][col1].getColor() == color){

							// チェック状況一覧に反映。削除候補にする
							checkedDeletes[row1][col1] = CHECK_DELETE;

							// 削除候補一覧に追加
							deleteKohoLst.add(new int[]{row1, col1});

						// 同じ色でない場合、
						}else{

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						}

					} // end if 未チェックの場合

				} // end if 一番上ではない場合

				// 下

				// 一番下ではない場合、
				if(row0 < BLOCK_ROW_COUNT - 1){

					// 位置情報
					int row1 = row0 + 1;
					int col1 = col0;

					// 未チェック かつ 削除ブロック一覧に登録されていない場合、
					if(checkedDeletes[row1][col1] == CHECK_NO_CHECK && deleteBlocks[row1][col1] == CHECK_NO_CHECK){

						// スプライトブロックが存在しない場合、
						if(sprBlocks[row1][col1] == null){

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 既に削除段階にある場合、
						}else if(sprBlocks[row1][col1].getDeleteStage() != -1){

							// チェック状況一覧に反映。残す。
							// （と、いうか、別のところで消える）
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						// 同じ色の場合、
						}else if(sprBlocks[row1][col1].getColor() == color){

							// チェック状況一覧に反映。削除候補にする
							checkedDeletes[row1][col1] = CHECK_DELETE;

							// 削除候補一覧に追加
							deleteKohoLst.add(new int[]{row1, col1});

						// 同じ色でない場合、
						}else{

							// チェック状況一覧に反映。残す。
							checkedDeletes[row1][col1] = CHECK_LEAVE;

						}

					} // end if 未チェックの場合

				} // end if 一番下ではない場合

			} // end for 削除候補一覧の件数分、ループ

			// 削除候補一覧が３以上の場合、
			if(deleteKohoLst.size() >= 3){

				// 削除フラグを削除ありにする
				flgDelete = true;

				// 削除候補一覧の件数分、ループ
				for(int j = 0; j < deleteKohoLst.size(); j++){

					// 削除候補を取得
					row = deleteKohoLst.get(j)[0];
					col = deleteKohoLst.get(j)[1];

					// 削除ブロック一覧に削除を設定
					deleteBlocks[row][col] = CHECK_DELETE;

				} // end for 削除候補一覧の件数分、ループ

			} // end if 削除候補一覧が３以上の場合

		} // end for チェックブロック一覧の件数分、ループ

		// 削除ありの場合、
		if(flgDelete){

			// 削除スプライトブロック一覧を生成
			ArrayList<KSpriteBlock01> deleteSprBlockLst	= new ArrayList<KSpriteBlock01>();

			// 行ループ
			for(int row = 0; row < BLOCK_ROW_COUNT; row++){

				// 列 ループ
				for(int col = 0; col < BLOCK_COL_COUNT; col++){

					// 削除の場合、
					if(deleteBlocks[row][col] == CHECK_DELETE){

						// スプライトブロックを取得
						KSpriteBlock01 sprBlock = sprBlocks[row][col];

						// 削除スプライトブロック一覧に追加
						deleteSprBlockLst.add(sprBlock);

						// 削除段階を設定
						sprBlock.setDeleteStage(0);

					} // end if 削除の場合

				} // end for 列ループ

			} // end for 行ループ

			// 削除スプライトブロック一覧をリターン
			return deleteSprBlockLst;

		} // end if 削除ありの場合、

		// nullでリターン
		return null;

	} // end check

	/**
	 * パネル取得.
	 * @return KPanel パネル
	 */
	public KPanel getKPanel(){

		// パネルをリターン
		return this.panel;

	}

	/**
	 * マウスイベント.
	 * <pre>
	 * マウスイベントによる制御を行います。
	 * </pre>
	 * @param me マウスイベント
	 */
	synchronized public void mouseEvent(MouseEvent me){

		// フェーズ分け
		switch(this.phase){

			// タイトル画面フェーズ
			case PHASE_TITLE:

				// クリック以外、
				if(me.getID() != MouseEvent.MOUSE_CLICKED){
					// リターン
					return;
				}

				// 音楽をストップ
				midiTitle.stop();

				// ステージ開始フェーズへ
				phase = PHASE_STAGE_START;

				break;

			// 通常フェーズ
			case PHASE_RUN:

				// 押した／ドラッグ／離した以外
				if(me.getID() != MouseEvent.MOUSE_PRESSED && me.getID() != MouseEvent.MOUSE_DRAGGED &&
								me.getID() != MouseEvent.MOUSE_RELEASED){
					// リターン
					return;
				}

				// 押した時、
				if(me.getID() == MouseEvent.MOUSE_PRESSED){

					// 左ボタン以外の場合、
					if(me.getButton() != MouseEvent.BUTTON1){
						// リターン
						return;
					}

					// 選んでいるブロック位置を初期化
					this.selectRow0 = -1;
					this.selectCol0 = -1;

					// 場所取得
					int y = me.getY();
					int x = me.getX();

					// ブロック位置取得
					int row = getRow(y);
					int col = getCol(x);

					// ブロックの場所以外の場合、
					if(row < 0 || row >= BLOCK_ROW_COUNT ||
									col < 0 || col >= BLOCK_COL_COUNT){
						// リターン
						return;
					}

					// 選んでいるブロック位置を保存
					this.selectRow0 = row;
					this.selectCol0 = col;

					// 選んでいるマウス位置を保存
					this.y0 = y;
					this.x0 = x;

				// ドラッグした時、
				}else if(me.getID() == MouseEvent.MOUSE_DRAGGED){

					// ブロック位置を初期化
					this.selectRow1 = -1;
					this.selectCol1 = -1;

					// 囲みを一旦解除
					panel.removeSprite(LAYOUT_SENTAKU, sprKakomuY);
					panel.removeSprite(LAYOUT_SENTAKU, sprKakomuX);

					// ブロック位置を保存していない場合、
					if(this.selectRow0 == -1){
						// リターン
						return;
					}

					// 場所取得
					int y = me.getY();
					int x = me.getX();

					// ブロック位置取得
					int row = getRow(y);
					int col = getCol(x);

					// 移動量（絶対値）を計算
					int amountX = Math.abs(this.x0 - x);
					int amountY = Math.abs(this.y0 - y);

					// X方向へ移動
					if(amountX >= amountY){

						// 選んでいるブロック位置と一緒の場合、
						if(this.selectCol0 == col){
							// リターン
							return;
						}

						// 左方向の場合、
						if(this.selectCol0 > col){

							// 選んでいるブロック位置が一番左の場合、
							if(this.selectCol0 == 0){
								// リターン
								return;
							}

							// 囲む
							sprKakomuX.setY(getYSprBlock(this.selectRow0));
							sprKakomuX.setX(getXSprBlock(this.selectCol0 - 1));
							panel.addSprite(LAYOUT_SENTAKU, sprKakomuX);

							// ブロック位置取得
							this.selectRow1 = this.selectRow0;
							this.selectCol1 = this.selectCol0 - 1;

						// 右方向の場合、
						}else{

							// 選んでいるブロック位置が一番右の場合、
							if(this.selectCol0 >= BLOCK_COL_COUNT - 1){
								// リターン
								return;
							}

							// 囲む
							sprKakomuX.setY(getYSprBlock(this.selectRow0));
							sprKakomuX.setX(getXSprBlock(this.selectCol0));
							panel.addSprite(LAYOUT_SENTAKU, sprKakomuX);

							// ブロック位置取得
							this.selectRow1 = this.selectRow0;
							this.selectCol1 = this.selectCol0 + 1;

						} // end if 右方向の場合

					// Y方向へ移動
					}else{

						// 選んでいるブロック位置と一緒の場合、
						if(this.selectRow0 == row){
							// リターン
							return;
						}

						// 上方向の場合、
						if(this.selectRow0 > row){

							// 選んでいるブロック位置が一番上の場合、
							if(this.selectRow0 == 0){
								// リターン
								return;
							}

							// 囲む
							sprKakomuY.setY(getYSprBlock(this.selectRow0 - 1));
							sprKakomuY.setX(getXSprBlock(this.selectCol0));
							panel.addSprite(LAYOUT_SENTAKU, sprKakomuY);

							// ブロック位置取得
							this.selectRow1 = this.selectRow0 - 1;
							this.selectCol1 = this.selectCol0;

						// 下方向の場合、
						}else{

							// 選んでいるブロック位置が一番下の場合、
							if(this.selectRow0 >= BLOCK_ROW_COUNT - 1){
								// リターン
								return;
							}

							// 囲む
							sprKakomuY.setY(getYSprBlock(this.selectRow0));
							sprKakomuY.setX(getXSprBlock(this.selectCol0));
							panel.addSprite(LAYOUT_SENTAKU, sprKakomuY);

							// ブロック位置取得
							this.selectRow1 = this.selectRow0 + 1;
							this.selectCol1 = this.selectCol0;

						} // end if 下方向の場合

					} // end if Y方向へ移動

				// 離した時、
				}else if(me.getID() == MouseEvent.MOUSE_RELEASED){

					// 左ボタン以外の場合、
					if(me.getButton() != MouseEvent.BUTTON1){
						// リターン
						return;
					}

					// 囲みを解除
					panel.removeSprite(LAYOUT_SENTAKU, sprKakomuY);
					panel.removeSprite(LAYOUT_SENTAKU, sprKakomuX);

					// ブロック位置を保存していない場合、
					if(this.selectRow0 == -1 || this.selectRow1 == -1){

						// ブロック位置を初期化
						this.selectRow0 = -1;
						this.selectCol0 = -1;
						this.selectRow1 = -1;
						this.selectCol1 = -1;

						// リターン
						return;
					}

					// 選んでいるブロック位置と一緒の場合、
					if(this.selectRow0 == this.selectRow1 && this.selectCol0 == this.selectCol1){

						// ブロック位置を初期化
						this.selectRow0 = -1;
						this.selectCol0 = -1;
						this.selectRow1 = -1;
						this.selectCol1 = -1;

						// リターン
						return;
					}

					// スプライトブロックを取得
					KSpriteBlock01 sprBlock0 = sprBlocks[selectRow0][selectCol0];
					KSpriteBlock01 sprBlock1 = sprBlocks[selectRow1][selectCol1];

					// スプライトブロックが存在しない場合、
					if(sprBlock0 == null || sprBlock1 == null){
						// リターン
						return;
					}

					// すでに削除中の場合、
					// もう一度消すと、おかしくなる
					if(sprBlock0.getDeleteStage() != -1 || sprBlock1.getDeleteStage() != -1){
						// リターン
						return;
					}

					// チェックブロック一覧を生成
					ArrayList<KSpriteBlock01> checkLst = new ArrayList<KSpriteBlock01>();

					// チェックブロック一覧に設定
					checkLst.add(sprBlock0);
					checkLst.add(sprBlock1);

					// ブロックの配置を変える
					blockChange(this.selectRow0, this.selectCol0, this.selectRow1, this.selectCol1);

					// 音楽をスタート
					waveClick.start();

					// チェック
					ArrayList<KSpriteBlock01> deleteSprBlockLst = check(checkLst);

					// 削除一覧が返却されてきた場合、
					if(deleteSprBlockLst != null){

						// 時間一覧に追加
						timeLst.add("0");

						// 削除スプライトブロック一覧一覧に追加
						deleteSprBlockLstLst.add(deleteSprBlockLst);

					// 削除一覧が返却されてこない場合、
					}else{

						// ブロックを元に戻す
						blockChange(this.selectRow0, this.selectCol0, this.selectRow1, this.selectCol1);

					} // end if 削除一覧が返却されてこない場合

					// ブロック位置を初期化
					this.selectRow0 = -1;
					this.selectCol0 = -1;
					this.selectRow1 = -1;
					this.selectCol1 = -1;

				} // end if

				break;

			default:
				break;

		} // end switch フェーズ分け

	} // end mouseEvent

	/**
	 * ブロック初期化.
	 */
	private void initBlocks(){

		// 行ループ
		for(int row = 0; row < BLOCK_ROW_COUNT; row++){

			// 列ループ
			for(int col = 0; col < BLOCK_COL_COUNT; col++){

				// スプライトブロックを生成
				KSpriteBlock01 sprBlock =  new KSpriteBlock01(panel, getNewBlock());

				// スプライトブロック一覧に設定
				sprBlocks[row][col] = sprBlock;

				// ブロック位置設定
				sprBlock.setY(POS_Y + row * 32);
				sprBlock.setX(POS_X + col * 32);

				// パネルにブロックを追加
				panel.addSprite(LAYOUT_BLK0, sprBlock);

			} // end for 列ループ

		} // end for 行ループ

	} // end initBlocks

	/**
	 * 新しいブロック色を取得.
	 * <pre>
	 * ステージを考慮したブロック色を返します。
	 * </pre>
	 * @return int 新しいブロック色
	 */
	private int getNewBlock(){
		return (int)(Math.random() * (stage + 2));
	}

	/**
	 * スコア加算.
	 * <pre>
	 * スコア加算する場合は、呼び出してください。
	 * </pre>
	 * @param scoreAdd 追加する点数
	 */
	public void addScore(int scoreAdd){

		// スコアを加算
		this.score = this.score + scoreAdd;

		// スコアを表示
		kTextScore.setText("SCORE : " + Integer.toString(score));

		// ハイスコアが更新された場合、
		if(this.score > this.scoreHigh){
		
			// ハイスコアを置き換え
			this.scoreHigh = this.score;

			// ハイスコアを表示
			kTextScoreHigh.setText("HIGH-SCORE : " + Integer.toString(scoreHigh));

		} // end if ハイスコアが更新された場合

		// ステージを進める場合、
		if(score > stage * 10000){

			// 制限時間を初期化
			limit = LIMIT_INIT;

			// ステージを進める
			stage++;

			// ステージを表示
			kTextStage.setText("STAGE : " + Integer.toString(stage));

		} // end ifステージを進める場合

	} // end addScore

	/**
	 * 行位置取得.
	 * <pre>
	 * パネル上の位置Yから行位置を取得します。
	 * ブロック位置ではなくても、仮想的な位置を返します。
	 * </pre>
	 * @param y 位置Y
	 * @return int 行位置
	 */
	private int getRow(int y){

		// 行位置をリターン
		return (y - POS_Y) / 32;

	} // end getRow

	/**
	 * 列位置取得.
	 * <pre>
	 * パネル上の位置Xから列位置を取得します。
	 * ブロック位置ではなくても、仮想的な位置を返します。
	 * </pre>
	 * @param x 位置X
	 * @return int 列位置
	 */
	private int getCol(int x){

		// 列位置をリターン
		return (x - POS_X) / 32;

	} // end getCol

	/**
	 * スプライトブロック位置Y取得.
	 * <pre>
	 * 行位置からパネル上の位置Yを取得します。
	 * </pre>
	 * @param row 行位置
	 * @return int 位置Y
	 */
	private int getYSprBlock(int row){

		// ブロック位置をリターン
		return POS_Y + row * 32;

	} // end getYSprBlock

	/**
	 * スプライトブロック位置X取得.
	 * <pre>
	 * 列位置からパネル上の位置Xを取得します。
	 * </pre>
	 * @param col 列位置
	 * @return int 位置X
	 */
	private int getXSprBlock(int col){

		// ブロック位置をリターン
		return POS_X + col * 32;

	} // end getXSprBlock

	/**
	 * ブロック入れ替え.
	 */
	private void blockChange(int row0, int col0, int row1, int col1){

		// スプライト位置を変更
		KSpriteBlock01 sprBlock = sprBlocks[row0][col0];
		sprBlocks[row0][col0] = sprBlocks[row1][col1];
		sprBlocks[row1][col1] = sprBlock;

		// スプライト座標を変更
		int x = sprBlocks[row0][col0].getX();
		sprBlocks[row0][col0].setX(sprBlocks[row1][col1].getX());
		sprBlocks[row1][col1].setX(x);

		int y = sprBlocks[row0][col0].getY();
		sprBlocks[row0][col0].setY(sprBlocks[row1][col1].getY());
		sprBlocks[row1][col1].setY(y);

	} // end blockChange

	/**
	 * マウスイベント.
	 */
	private class KCPMouseAdapter extends MouseAdapter {

		/**
		 * マウスクリック.
		 */
		public void mouseClicked(MouseEvent me){

			// スーパークラス呼び出し
			super.mouseClicked(me);

			// マウスイベント呼び出し
			KCtrlPZL.this.mouseEvent(me);

		} // end mouseClicked

		/**
		 * マウス押された.
		 */
		public void mousePressed(MouseEvent me){

			// スーパークラス呼び出し
//			super.mousePressed(me);

			// マウスイベント呼び出し
			KCtrlPZL.this.mouseEvent(me);

		} // end mousePressed

		/**
		 * マウス離された.
		 */
		public void mouseReleased(MouseEvent me){

			// スーパークラス呼び出し
//			super.mouseReleased(me);

			// マウスイベント呼び出し
			KCtrlPZL.this.mouseEvent(me);

		} // end mouseReleased

		/**
		 * ドラッグされた.
		 */
		public void mouseDragged(MouseEvent me){

			// スーパークラス呼び出し
//			super.mouseDragged(me);

			// マウスイベント呼び出し
			KCtrlPZL.this.mouseEvent(me);

		} // end mouseDragged

	} // end KCPMouseAdapter

}
